/********************************************************************
  
  Cachorro Eletrônico V3
  
  P/ Placa Arduino Nano 
  
  C/ PIR SENSOR MODULE e microfone de eletreto
  
  Reproduz arquivo de som gravado na flash do microcontrolador.
  Método PWM  p/ reproduzir .wav, 8bits, 8khz e mono.
  
  Autor: Cláudio Lários

  Data: 26/08/2019
  
  O arquivo 'include' contém os sons obtidos da internet
  devidamente convertido para uso nesta montagem.
  Para o disparo do som, usamos um microfone de eletreto
  ligado no pino A0.
  Um 'trimpot' de 5k permite ajustar o limiar de disparo
  pelo som ambiente.
  Também foi adicionado um módulo PIROELETRICO para detenção
  de movimento de pessoas, ligado ao pino D3. 
  Um led ligado ao pino D2 acende quando ocorre a reprodução.
  Também ocorrerá o acionamento de um rele. Este poderá
  ligar uma iluminação externa, por um tempo determinado
  neste programa.
  Este sketch se aproxima mais de um simulador de presença
  com reprodução de latidos. 
  Portanto seu objetivo será inibir a entrada  de intrusos na
  área que estiver instalado este dispositivo.  
  Amplificadores de som de maior potência podem ser ligados
  ao pino 6 caso deseje maior volume no som.

  Obs.Testado somente no Arduino Nano
  Outras placas talvez necessitem ajustar o programa e
  hardware para funcionarem.

  Uso didático

  Blog Picsource  

 ******************************************************************/ 

#include "SOM_CACHORRO_LATINDO.c"    //som de um cachorro 'bravo' latindo
// do tipo mono, 8bits, amostragem feita em 8 kytes.

//definição de pinos e variáveis globais
int len = 27000;//quantidade de bytes do arquivo acima (veja cabeçalho)
int somPinA = 6;// pino 6 é a saída de som (PWM)
int LED = 2; //pino 2 Led indicador acionamento do rele
int sensorPir = 3;//pino 3 ligado ao módulo sensor piroelétrico
int mem_adress;//armazena endereço sequencial do arquivo acessado
long timerLuz=0;//contador de tempo que rele fica ligado

//defines para opções de funcionamento
#define TEMPO_LUZ_ACESA 5  //em minutos (max=8500 min)  
#define PATAMAR 800 //sensibilidade do microfone aos sons ambientes (0 a 1023)
#define REPETIR_MIC  2  //repete gravação por N vezes seguidas (p/MICROFONE)
#define REPETIR_PIR  15  //repete gravação por N vezes seguidas (p/PIR)
#define NLATIDA0N 4   //após 4 repetiçoes liga rele (p/ uma luz externa)
#define TIPO_CACHORRO 4 //cachorro grande use 4 e cachorro pequeno use 3

//defines auxiliares 
#define FADJTIMER  500000  //fator para obter aproximadamente 1 minuto 
#define TLUZON  ( FADJTIMER * TEMPO_LUZ_ACESA) 
#define LIGADO HIGH
#define DESLIGADO LOW                                          

//rotina do setup
void setup() {
 
      pinMode(somPinA, OUTPUT);//pino 6 como saída do PWM 
      pinMode(LED, OUTPUT);//pino 2 como saída do led + rele
      pinMode(sensorPir, INPUT);//pino 3 como entrada do módulo sensor PIR
 
  //inicializa PWM com frequência de ~32khz 
  //Obs. não use funções de tempo (millis(),delay(), delayMicroseconds())
  //pois estamos alterando todo o funcionamento do timer0 que gerencia
  //tais funções acima citadas.
  
     TCCR0B= B00000001;//canal A pino 6 como saída PWM
     TCCR0A=0B10000011; //modo fast PWM 
     digitalWrite(LED,DESLIGADO);//inicia com led e rele desligado
}



//rotina do loop main
void loop() {
  
      
      int nrepetir=REPETIR_MIC;//carrega número de vezes que será repetido o som p/ MICROFONE    
      int micValue= analogRead(A0);//le a tensão do microfone  
      int npat=PATAMAR;
      if(digitalRead(sensorPir)){
        nrepetir=REPETIR_PIR;//carrega numero de vezes quando disparo for pelo PIR MODULE
        npat=1023;//equivale a forçar a execução da rotina 'if(micValue<npat){'   
      }
      if(micValue<npat){//verifica se micValue é menor que o do patamar de disparo
    //  digitalWrite(LED,LIGADO);//se sim, acende o led
      for(int a=0;a<nrepetir;a++)// entra na rotina de repetição por x vezes
      {
        if(a>=NLATIDA0N){
          digitalWrite(LED,LIGADO);//se sim, acende o led e aciona rele
          timerLuz=TLUZON; //carrega valor de temporização enquanto na rotina de repetição
          //obs. se for usado 'if(a==NLATIDA0N){' contará tempo a partir da 4º repetição
          //Mas se for usado 'if(a>=NLATIDA0N){'  contará tempo somente ao fim da reprodução
        }
           
        for (mem_adress = 0; mem_adress < len; mem_adress++)//inicia a reprodução  
        {    
           OCR0A =  pgm_read_word_near(sound_data1 + mem_adress); //obtem som gravado na flash         
           for(int i=0;i<TIPO_CACHORRO;i++){//OCR0A é mantido N vezes com o mesmo valor 
             while((TIFR0&1)==0); //aguarda fim do ciclo PWM   
             TIFR0&=0B11111110;//reseta flag TOV0  
             if(digitalRead(sensorPir)){
                nrepetir=REPETIR_PIR; //altera a quantidade que será repetido a reprodução
                npat=1023;//equivale a forçar a execução da rotina 'if(micValue<npat){'    
             }
           }   
        }//for (mem_adress = 0; mem_adress < ...   
      } //for(a=0;a<REPETIR_X;... 
      
        
        
     }//if(micValue>... 
     
     if(timerLuz>0)timerLuz--;//decrementa contador de tempo
     if(timerLuz==0) {
        digitalWrite(LED,DESLIGADO); //se finalizou o tempo desliga led + rele        
     }
   
  }//void loop  

    
